@@ -1,7 +1,7 @@
 /* unicode.c
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: unicode.c,v 1.11 2002/03/12 11:33:55 simon Exp $
+ *     $Id: unicode.c,v 1.14 2002/07/17 05:03:57 josh Exp $
  *  Overview:
  *     This defines the US-ASCII character type routines.
  *  Data Structure and Algorithms:
@@ -15,22 +15,25 @@
 static CHARTYPE_TRANSCODER
 unicode_transcode_from(const char *from)
 {
-    return from ? (CHARTYPE_TRANSCODER)NULL : (CHARTYPE_TRANSCODER)NULL;        /* NOTE: This cheat silences unused var warnings on gcc */
+    /* NOTE: This cheat silences unused var warnings on gcc */
+    return from ? (CHARTYPE_TRANSCODER)NULL : (CHARTYPE_TRANSCODER)NULL; 
 }
 
 static CHARTYPE_TRANSCODER
 unicode_transcode_to(const char *to)
 {
-    return to ? (CHARTYPE_TRANSCODER)NULL : (CHARTYPE_TRANSCODER)NULL;  /* NOTE: This cheat silences unused var warnings on gcc */
+    /* NOTE: This cheat silences unused var warnings on gcc */
+    return to ? (CHARTYPE_TRANSCODER)NULL : (CHARTYPE_TRANSCODER)NULL;
 }
 
-static BOOLVAL
+static Parrot_Int
 unicode_is_digit(UINTVAL c)
 {
-    return (BOOLVAL)(isdigit((int)c) ? 1 : 0);  /* FIXME - Other code points are also digits */
+    /* FIXME - Other code points are also digits */
+    return (INTVAL)(isdigit((int)c) ? 1 : 0);  
 }
 
-static INTVAL
+static Parrot_Int
 unicode_get_digit(UINTVAL c)
 {
     return c - '0';             /* FIXME - many more digits than this... */
