@@ -2,7 +2,7 @@
  * disassemble.c
  *
  * CVS Info
- *    $Id: disassemble.c,v 1.1 2002/06/01 02:52:35 jgoff Exp $
+ *    $Id: disassemble.c,v 1.2 2002/06/07 01:31:57 josh Exp $
  * Overview:
  *    Parrot disassembler
  * History:
@@ -24,7 +24,7 @@ int
 main(int argc, char *argv[])
 {
     Parrot_Interp interpreter;
-    char *filename,*disassemble;
+    char *filename, *disassemble;
     Parrot_PackFile pf;
 
     interpreter = Parrot_new();
@@ -35,9 +35,8 @@ main(int argc, char *argv[])
 
     Parrot_init(interpreter);
 
-    if (argc != 2)
-    {
-        fprintf(stderr,"Usage: disassemble programfile \n");
+    if (argc != 2) {
+        fprintf(stderr, "Usage: disassemble programfile \n");
         exit(1);
     }
 
