@@ -1,7 +1,7 @@
 /* chartype.h
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: chartype.h,v 1.10 2002/04/17 03:50:28 josh Exp $
+ *     $Id: chartype.h,v 1.12 2002/06/25 20:06:43 dmlloyd Exp $
  *  Overview:
  *     This is the api header for the string character type subsystem
  *  Data Structure and Algorithms:
@@ -27,7 +27,7 @@ struct parrot_chartype_t {
     const char *default_encoding;
     Parrot_CharType_Transcoder (*transcode_from)(const char *from);
     Parrot_CharType_Transcoder (*transcode_to)(const char *to);
-    Parrot_Bool (*is_digit)(Parrot_UInt c);
+    Parrot_Int (*is_digit)(Parrot_UInt c);
     Parrot_Int (*get_digit)(Parrot_UInt c);
 };
 
@@ -36,8 +36,8 @@ struct parrot_chartype_t {
 #ifdef PARROT_IN_CORE
 
 #define CHARTYPE struct parrot_chartype_t
-const CHARTYPE *
-chartype_lookup_index(INTVAL n);
+
+const CHARTYPE * chartype_lookup_index(INTVAL n);
 
 typedef Parrot_CharType_Transcoder CHARTYPE_TRANSCODER;
 
