@@ -7,11 +7,14 @@ use vars qw(@steps);
 @steps=qw(
 	init/manifest.pl
 	init/data.pl
+	init/miniparrot.pl
 	init/hints.pl
 	inter/progs.pl
 	inter/types.pl
 	inter/ops.pl
 	inter/exp.pl
+	inter/pmc.pl
+    auto/alignptrs.pl
 	auto/headers.pl
 	auto/sizes.pl
 	auto/byteorder.pl
@@ -26,6 +29,7 @@ use vars qw(@steps);
 	gen/makefiles.pl
 	gen/platform.pl
 );
+#	auto/alignptrs.pl
 
 sub runsteps {
 	shift;
@@ -47,7 +51,7 @@ sub runsteps {
 		}
 
 		print "done." unless m{^inter/} && $args{ask};
-	}
+	}	
 }
 
-1;
\ No newline at end of file
+1;
