@@ -7,7 +7,7 @@
 ** This program is free software. It is subject to the same
 ** license as Parrot itself.
 **
-** $Id: packfile.c,v 1.49 2002/06/01 01:52:12 josh Exp $
+** $Id: packfile.c,v 1.52 2002/06/12 22:58:29 dmlloyd Exp $
 **
 ** History:
 **  Rework by Melvin; new bytecode format, make bytecode portable.
@@ -101,7 +101,7 @@ byteorder if needed.
 
 ***************************************/
 
-INLINE opcode_t
+opcode_t
 PackFile_fetch_op(struct PackFile *pf, opcode_t *stream) {
     if(pf->fetch_op == NULL)
         return *stream;
@@ -122,7 +122,7 @@ byteorder if needed.
 
 ***************************************/
 
-INLINE INTVAL
+INTVAL
 PackFile_fetch_iv(struct PackFile *pf, opcode_t *stream) {
     if(pf->fetch_iv == NULL)
         return *stream;
@@ -140,7 +140,7 @@ byteorder if needed.
 
 ***************************************/
 
-INLINE FLOATVAL
+FLOATVAL
 PackFile_fetch_nv(struct PackFile *pf, opcode_t *stream) {
     /* When we have alignment all squared away we don't need
      * to use memcpy() for native byteorder.
@@ -181,12 +181,12 @@ void PackFile_assign_transforms(struct PackFile *pf) {
         /* FIXME: Use the float_type from bytecode header to decide vtable */
         pf->fetch_nv = fetch_buf_be_8;
     }
-#if TRACE_PACKFILE
+#  if TRACE_PACKFILE
     else {
         fprintf(stderr, "header->byteorder [%d] native byteorder [%d]\n",
             pf->header->byteorder, PARROT_BIGENDIAN);
     }
-#endif
+#  endif
 #endif    
 }
 
@@ -230,7 +230,7 @@ PackFile_destroy(struct PackFile *pf)
 }
 
 /* Internal function to check segment_size % sizeof(opcode_t) */
-static BOOLVAL
+static INTVAL
 PackFile_check_segment_size(opcode_t segment_size, const char *debug)
 {
 #if TRACE_PACKFILE
@@ -469,7 +469,7 @@ Returns one (1) if everything is OK, else zero (0).
 
 ***************************************/
 
-BOOLVAL
+INTVAL
 PackFile_FixupTable_unpack(struct PackFile_FixupTable *self, opcode_t *packed,
                            opcode_t packed_size)
 {
@@ -535,7 +535,7 @@ Returns one (1) if everything is OK, else zero (0).
 
 ***************************************/
 
-BOOLVAL
+INTVAL
 PackFile_ConstTable_unpack(struct Parrot_Interp *interpreter,
                             struct PackFile * pf,
                             struct PackFile_ConstTable *self, opcode_t *packed,
@@ -576,7 +576,7 @@ PackFile_ConstTable_unpack(struct Parrot_Interp *interpreter,
     }
 
     for (i = 0; i < self->const_count; i++) {
-        BOOLVAL rc;
+        INTVAL rc;
 #if TRACE_PACKFILE
         printf("PackFile_ConstTable_unpack(): Unpacking constant %ld...\n", i);
 #endif
@@ -742,7 +742,7 @@ Returns one (1) if everything is OK, else zero (0).
 
 ***************************************/
 
-BOOLVAL
+INTVAL
 PackFile_Constant_unpack(struct Parrot_Interp *interpreter,
                          struct PackFile *pf,
                          struct PackFile_Constant *self, opcode_t *packed,
@@ -751,7 +751,7 @@ PackFile_Constant_unpack(struct Parrot_Interp *interpreter,
     opcode_t *cursor;
     opcode_t type;
     opcode_t size;
-    BOOLVAL rc = 1;
+    INTVAL rc = 1;
 
     UNUSED(packed_size);
 
@@ -807,7 +807,7 @@ Returns one (1) if everything is OK, else zero (0).
 
 ***************************************/
 
-BOOLVAL
+INTVAL
 PackFile_Constant_unpack_number(struct PackFile * pf, struct PackFile_Constant *self,
                                 opcode_t *packed, opcode_t packed_size)
 {
@@ -867,7 +867,7 @@ Returns one (1) if everything is OK, else zero (0).
 
 ***************************************/
 
-BOOLVAL
+INTVAL
 PackFile_Constant_unpack_string(struct Parrot_Interp *interpreter,
                                 struct PackFile * pf,
                                 struct PackFile_Constant *self,
