@@ -2,7 +2,7 @@
  * pdb.c
  *
  * CVS Info
- *    $Id: pdb.c,v 1.1 2002/06/01 02:52:35 jgoff Exp $
+ *    $Id: pdb.c,v 1.2 2002/06/07 01:31:57 josh Exp $
  * Overview:
  *    The Parrot debugger
  * History:
@@ -37,9 +37,8 @@ main(int argc, char *argv[])
 
     Parrot_init(interpreter);
 
-    if (argc != 2)
-    {
-        fprintf(stderr,"Usage: pdb programfile\n");
+    if (argc != 2) {
+        fprintf(stderr, "Usage: pdb programfile\n");
         exit(1);
     }
 
@@ -64,7 +63,7 @@ main(int argc, char *argv[])
 void
 PDB_printwelcome()
 {
-    fprintf(stderr,"Parrot Debugger 0.0.1\n");
+    fprintf(stderr, "Parrot Debugger 0.0.1\n");
 }
 
 /*
