@@ -1,7 +1,7 @@
 /* string.c
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: string.c,v 1.79 2002/05/20 18:54:23 grunblatt Exp $
+ *     $Id: string.c,v 1.82 2002/07/18 04:29:39 mongo Exp $
  *  Overview:
  *     This is the api definitions for the string subsystem
  *  Data Structure and Algorithms:
@@ -15,6 +15,8 @@
 static const CHARTYPE *string_native_type;
 static const CHARTYPE *string_unicode_type;
 
+#define EXTRA_SIZE 4
+
 /* Basic string stuff - creation, enlargement, destruction, etc. */
 
 /*=for api string string_init
@@ -27,6 +29,73 @@ string_init(void)
     string_unicode_type = chartype_lookup("unicode");
 }
 
+/*=for api string string_append
+ * Take in two strings and append the second string to the first
+ */
+STRING *
+string_append(struct Parrot_Interp *interpreter, STRING *a,
+              STRING *b, UINTVAL Uflags)
+{
+    STRING *temp;
+    UNUSED(Uflags);
+
+    /* If B isn't real, we just bail */
+    if (b == NULL || b->strlen == 0) {
+        return a;
+    }
+
+    /* Is A real? */
+    if (a != NULL && a->strlen != 0) {
+        /* If the destination's constant, then just fall back to
+           string_concat */
+        if (a->flags & BUFFER_constant_FLAG) {
+            return string_concat(interpreter, a, b, Uflags);
+        }
+        /* First, make sure B is the same type as A, transcoding
+           if we need to */
+        if (a->type != b->type || a->encoding != b->encoding) {
+            b = string_transcode(interpreter, b, a->encoding, a->type,
+                                 NULL);
+        }
+        /* make sure A's big enough for both */
+        if (a->buflen < a->bufused + b->bufused) {
+            a = string_grow(interpreter, a, ((a->bufused + b->bufused)
+                            - a->buflen) + EXTRA_SIZE);
+        }
+        /* Tack B on the end of A */
+        mem_sys_memcopy((void *)((ptrcast_t)a->bufstart + a->bufused),
+                        b->bufstart, b->bufused);
+        a->bufused += b->bufused;
+        a->strlen += b->strlen;
+        return a;
+    }
+    else {
+        /* A isn't real. Does it exist at all? */
+        if (a != NULL) {
+            if (a->flags & BUFFER_constant_FLAG) {
+                return string_concat(interpreter, a, b, Uflags);
+            }
+            /* There's at least a string header for A. Make it a copy
+               of B */
+            if (a->buflen < b->bufused) {
+                a = string_grow(interpreter, a, b->bufused + EXTRA_SIZE);
+            }
+            a->flags = b->flags;
+            a->flags &= ~(UINTVAL)BUFFER_constant_FLAG;
+            a->bufused = b->bufused;
+            a->strlen = b->strlen;
+            a->encoding = b->encoding;
+            a->type = b->type;
+            a->language = b->language;
+            memcpy(a->bufstart, b->bufstart, b->bufused);
+            return a;
+        }
+    }
+    /* If we got here, A was NULL. So clone B. */
+    return string_copy(interpreter, b);
+}
+
+
 /*=for api string string_make
  * allocate memory for the string, copy information into it
  * and compute its string length
@@ -271,14 +340,12 @@ string_concat(struct Parrot_Interp *interpreter, STRING *a,
             if (a->type != b->type || a->encoding != b->encoding) {
                 b = string_transcode(interpreter, b, a->encoding, a->type,
                                      NULL);
-                b->flags |= BUFFER_neonate_FLAG;
             }
             result = string_make(interpreter, NULL, a->bufused + b->bufused,
                                  a->encoding, 0, a->type);
             mem_sys_memcopy(result->bufstart, a->bufstart, a->bufused);
             mem_sys_memcopy((void *)((ptrcast_t)result->bufstart + a->bufused),
-                            b->bufstart, b->bufused);
-            b->flags &= ~(UINTVAL)BUFFER_neonate_FLAG;
+                             b->bufstart, b->bufused);
             result->strlen = a->strlen + b->strlen;
             result->bufused = a->bufused + b->bufused;
         }
@@ -429,7 +496,6 @@ string_replace(struct Parrot_Interp *interpreter, STRING *src,
     if (rep->encoding != src->encoding || rep->type != src->type) {
         rep = string_transcode(interpreter, rep, src->encoding, src->type, 
                                NULL);
-        rep->flags |= BUFFER_neonate_FLAG;
     }
 
     /* abs(-offset) may not be > strlen-1 */
@@ -518,7 +584,6 @@ string_replace(struct Parrot_Interp *interpreter, STRING *src,
         src->bufused += diff;
         (void)string_compute_strlen(src);
     } 
-    rep->flags &= ~(UINTVAL)BUFFER_neonate_FLAG;
 
     /* src is modified, now return the original substring */    
     return dest;
@@ -576,10 +641,8 @@ string_compare(struct Parrot_Interp *interpreter, STRING *s1,
     if (s1->type != s2->type || s1->encoding != s2->encoding) {
         s1 = string_transcode(interpreter, s1, NULL, string_unicode_type,
                               NULL);
-        s1->flags |= BUFFER_neonate_FLAG;
         s2 = string_transcode(interpreter, s2, NULL, string_unicode_type,
                               NULL);
-        s2->flags |= BUFFER_neonate_FLAG;
     }
 
     s1start = s1->bufstart;
@@ -596,8 +659,6 @@ string_compare(struct Parrot_Interp *interpreter, STRING *s1,
         s1start = s1->encoding->skip_forward(s1start, 1);
         s2start = s2->encoding->skip_forward(s2start, 1);
     }
-    s1->flags &= ~(UINTVAL)BUFFER_neonate_FLAG;
-    s2->flags &= ~(UINTVAL)BUFFER_neonate_FLAG;
 
     if (cmp == 0 && s1start < s1end)
         cmp = 1;
@@ -608,7 +669,7 @@ string_compare(struct Parrot_Interp *interpreter, STRING *s1,
 }
 
 /* A string is "true" if it is equal to anything but "" and "0" */
-BOOLVAL
+INTVAL
 string_bool(const STRING *s)
 {
     INTVAL len;
@@ -617,13 +678,15 @@ string_bool(const STRING *s)
     }
 
     len = string_length(s);
-
+	
     if (len == 0) {
         return 0;
     }
 
     if (len == 1) {
+
         UINTVAL c = s->encoding->decode(s->bufstart);
+
         if (s->type->is_digit(c) && s->type->get_digit(c) == 0) {
             return 0;
         }
@@ -654,7 +717,7 @@ string_to_int(const STRING *s)
         const char *start = s->bufstart;
         const char *end = start + s->bufused;
         int sign = 1;
-        BOOLVAL in_number = 0;
+        INTVAL in_number = 0;
 
         while (start < end) {
             UINTVAL c = s->encoding->decode(start);
@@ -694,11 +757,11 @@ string_to_num(const STRING *s)
         const char *start = s->bufstart;
         const char *end = start + s->bufused;
         int sign = 1;
-        BOOLVAL seen_dot = 0;
-        BOOLVAL seen_e = 0;
+        INTVAL seen_dot = 0;
+        INTVAL seen_e = 0;
         int exp_sign = 0;
-        BOOLVAL in_exp = 0;
-        BOOLVAL in_number = 0;
+        INTVAL in_exp = 0;
+        INTVAL in_number = 0;
         FLOATVAL exponent = 0;
         INTVAL fake_exponent = 0;
 
